/* @flow */
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var spawn = require('child_process').spawn;
var _ = require('highland');
var sanitize = require('sanitize-filename');
var fs = require('fs');
var Shapefile = require('./shapefile');
var Cmd = require('./ogr-cmd');

function createStream(format, options) {
  options.input = options.path + '/layer.vrt';
  var tempVrt = fs.createWriteStream(options.input);
  var output = _.pipeline(function (stream) {
    // we set up this passthru so a stream can be immediately returned
    // even tho data will not start coming int until the vrt finish event
    var pass = _();
    // first write the incoming text as a temporary VRT file on disk
    stream.pipe(tempVrt).on('error', function (e) {
      return output.emit('error', e);
    })
    // when the VRT is on disk we can crank up OGR
    .on('finish', function () {
      options.name = options.name ? sanitize(options.name) : 'output';
      var ogrStream = format === 'zip' ? createShp(options) : spawnOgr(format, options);
      return ogrStream.on('error', function (e) {
        return output.emit('error', e);
      }).pipe(pass);
    });
    return pass;
  }).on('properties', function (p) {
    return _extends(options, p);
  });

  return output;
}

function spawnOgr(format, options) {
  var output = _();
  var cmd = Cmd.create(format, options);
  output.emit('log', { level: 'info', message: 'Executing: OGR2OGR ' + cmd.join(' ') });
  var ogr = spawn('ogr2ogr', cmd).on('close', function (c) {
    if (c > 0) output.emit('error', new Error('OGR Failed'));
  });
  ogr.stderr.on('data', function (data) {
    var msg = data.toString();
    // Error 1: GeoJSON parsing error
    // Error 4: Failed to read GeeoJSON
    // Error 6: debug message that can be ignored
    if (msg.match(/ERROR\s[^6]/)) output.emit('error', new Error(msg));
  });
  return ogr.stdout.pipe(output);
}

function createShp(options) {
  var output = _();
  return Shapefile.createStream(options).on('error', function (e) {
    return output.emit('error', e);
  }).on('log', function (l) {
    return output.emit('log', l);
  }).pipe(output);
}

module.exports = { createStream: createStream };